import nbformat
import pytest

import jupytext
from jupytext.compare import compare_notebooks

from .utils import list_notebooks


@pytest.mark.parametrize("nb_file", list_notebooks("ipynb_py"))
def test_identity_source_write_read(nb_file):
    """Test that writing the notebook with jupytext, and read again,
    is the same as removing outputs"""

    with open(nb_file) as fp:
        nb1 = nbformat.read(fp, as_version=4)

    py = jupytext.writes(nb1, "py")
    nb2 = jupytext.reads(py, "py")

    compare_notebooks(nb2, nb1)



"""Read and write Jupyter notebooks as text files"""

from .formats import NOTEBOOK_EXTENSIONS, get_format_implementation, guess_format
from .jupytext import read, reads, write, writes
from .reraise import reraise
from .version import __version__

try:
    from .contentsmanager import build_jupytext_contents_manager_class
except ImportError as err:
    build_jupytext_contents_manager = reraise(err)

try:
    from .contentsmanager import TextFileContentsManager
except ImportError as err:
    TextFileContentsManager = reraise(err)


def _jupyter_labextension_paths():  # pragma: no cover
    return [{"src": "labextension", "dest": "jupyterlab-jupytext"}]


def load_jupyter_server_extension(app):  # pragma: no cover
    """Use Jupytext's contents manager"""
    if hasattr(app.contents_manager_class, "formats"):
        app.log.info(
            "[Jupytext Server Extension] NotebookApp.contents_manager_class is "
            "(a subclass of) jupytext.TextFileContentsManager already - OK"
        )
        return

    # The server extension call is too late!
    # The contents manager was set at NotebookApp.init_configurables

    # Let's change the contents manager class
    app.log.info(
        "[Jupytext Server Extension] Deriving a JupytextContentsManager "
        "from {}".format(app.contents_manager_class.__name__)
    )
    app.contents_manager_class = build_jupytext_contents_manager_class(
        app.contents_manager_class
    )

    try:
        # And rerun selected init steps from https://github.com/jupyter/notebook/blob/
        # 132f27306522b32fa667a6b208034cb7a04025c9/notebook/notebookapp.py#L1634-L1638

        # app.init_configurables()
        app.contents_manager = app.contents_manager_class(parent=app, log=app.log)
        app.session_manager.contents_manager = app.contents_manager

        # app.init_components()
        # app.init_webapp()
        app.web_app.settings["contents_manager"] = app.contents_manager

        # app.init_terminals()
        # app.init_signal()

    except Exception:
        app.log.error(
            """[Jupytext Server Extension] An error occurred. Please deactivate the server extension with
    jupyter serverextension disable jupytext
and configure the contents manager manually by adding
    c.NotebookApp.contents_manager_class = "jupytext.TextFileContentsManager"
to your .jupyter/jupyter_notebook_config.py file.
"""
        )
        raise


def _jupyter_nbextension_paths():  # pragma: no cover
    """Allows commands like
    jupyter nbextension install --py jupytext
    jupyter nbextension enable --py jupytext
    jupyter labextension install jupyterlab-jupytext"""
    return [
        dict(
            section="notebook",
            # the path is relative to the `jupytext` directory
            src="nbextension",
            # directory in the `nbextension/` namespace
            dest="jupytext",
            # _also_ in the `nbextension/` namespace
            require="jupytext/index",
        )
    ]


__all__ = [
    "read",
    "write",
    "writes",
    "reads",
    "NOTEBOOK_EXTENSIONS",
    "guess_format",
    "get_format_implementation",
    "TextFileContentsManager",
    "__version__",
]



import nbformat
import pytest

import jupytext

from .utils import list_notebooks


@pytest.mark.skipif(nbformat.__version__ <= "5.7", reason="normalize is not available")
@pytest.mark.parametrize("nb_file", list_notebooks("all", skip="(invalid|pyc)"))
def test_sample_notebooks_are_normalized(nb_file):
    nb = jupytext.read(nb_file)

    changes, normalized_nb = nbformat.validator.normalize(nb)
    nbformat.validate(normalized_nb)

    if changes:  # pragma: no cover
        with open(nb_file, "w") as fp:
            jupytext.write(normalized_nb, fp)

        assert not changes

    nbformat.validate(nb)



# ---
# jupyter:
#   jupytext:
#     cell_markers: '{{{,}}}'
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# {{{
# %%time

print('asdf')
# }}}

# Thanks for jupytext!


